/*
m_midi_ratio - return (sub)harmonic ratios based on a linear control input

DESCRIPTION
This opcode returns only harmonic and subharmonic ratios, i.e. 1/6, 1/5, ...,
1, 2, 3, etc. The output will be stepped (quantised). This is designed to
work with MIDI controllers as input, but any control signal which can grow
large enough, because only integral values will map to harmonic ratios.

SYNTAX
kRatio m_midi_ratio kInput, iMinRatio, iMaxRatio

INITIALIZATION
iMinRatio, iMaxRatio - Lowest and highest ratios (will be quantised to harmonic
	ratios, i.e. 1/n or n

PERFORMANCE
kRatio - Resulting harmonic ratio value
kInput - Input controller, can be a MIDI CC value or any other control signal.
	A kInput of 0 maps to the lowest harmonic ratio.

CREDITS
Author: Jeanette C.
*/

opcode m_midi_ratio, k, kii
	kInput, iMin, iMax xin

	; Initialise parameters
	kRatio init 1
	
	; Determine quantised lowest and highest ratios
	if (iMin <1) then
		iMinValue = -1 * round(1/iMin) ; negative value to denote subharmonic
		iMinBound = iMinValue + 1
		iOneRatio = -1 * (iMinValue + 1)
	else
		iMinValue = round(iMin)
		iMinBound = iMinValue
		iOneRatio = -1 * (iMinValue - 1)
	endif

	if (iMax <1) then
		iMaxValue = -1 * round(1/iMax) ; negative value to denote subharmonic
		iMaxBound = iMaxValue + 1
	else
		iMaxValue = round(iMax)
		iMaxBound = iMaxValue
	endif

	; Maximum accepted input value (upper limit)
	iMaxInput = iMaxBound - iMinBound - .501

	; Limit and round input
	kRealInput = round(limit:k(kInput, 0, iMaxInput)) - iOneRatio

	if (kRealInput >= 0) then
		kRatio = kRealInput + 1
	else
		kRatio = 1/((kRealInput -1) * -1)
	endif

	; Output the ratio
	xout(kRatio)
endop
